<?php
namespace XingWai;

class Tools
{
	private function __construct()
	{
	}

	static public function log($log = '', $functionName = 'default')
	{
		logActivity('Geeba Logs ( 函数名称: \'' . $functionName . '\' | 日志内容: \'' . $log . '\' )', 0);
	}

	static public function getRandomMD5()
	{
		return md5(mt_rand(1, 10) . time());
	}

	static public function getRandomString($size = '')
	{
		$size = (empty($size) ? 8 : (int) $size);

		if ($size < 4) {
			throw new \Exception('随机字符串长度低于 4 位');
		}

		$result = '';
		$count = floor($size / 32);
		$i = 0;

		while ($i <= $count) {
			$result = $result . self::getRandomMD5();
			++$i;
		}

		return substr($result, 0, (int) $size);
	}

	/**
     * 返回一个 A 标签
     * @param string $url 传入 URL 地址
     * @param string $name 传入链接的名称
     * @param string $css 传入额外的 css
     * @return string
     */
	static public function getLinkTag($url = '', $name = '', $css = '')
	{
		$css = (empty($css) ? '' : 'style="' . $css . '"');
		return (string) '<a href="' . $url . '" target="_blank" ' . $css . ' >' . $name . '</a>';
	}

	/**
     * 获取网页内容
     * @param array $setting 包含有 url / timeout / user-agent / post 等键
     * @return mixed
     * @throws Exception 在无法取得网页内容或网页返回的内容并非 JSON 格式时会报错
     */
	static public function getWebContent(array $setting = array())
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $setting['url']);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_TIMEOUT, empty($setting['timeout']) ? 5 : $setting['timeout']);
		curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
		curl_setopt($ch, CURLOPT_FRESH_CONNECT, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_USERAGENT, empty($setting['user-agent']) ? 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.12; rv:51.0) Gecko/20100101 Firefox/51.0' : $setting['user-agent']);

		if (!empty($setting['post'])) {
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($setting['post']));
		}

		$data = curl_exec($ch);

		if ($setting['utf8']) {
			$data = mb_convert_encoding($data, 'utf8', 'gbk');
		}

		if (curl_error($ch)) {
			throw new \Exception('无法取得网页内容, 错误码: #' . curl_errno($ch) . ' 错误原因: ' . curl_error($ch));
		}

		curl_close($ch);

		if (empty($data)) {
			throw new \Exception('取得的网页内容为空');
		}

		if (!empty($setting['json'])) {
			$data = json_decode($data, true);

			if ($data == 'null') {
				throw new \Exception('网页返回内容并非 JSON 格式');
			}
		}

		return $data;
	}

	static public function getSystemURL()
	{
		if (empty($GLOBALS['CONFIG'])) {
			$result = \WHMCS\Database\Capsule::table('tblconfiguration')->where('setting', 'SystemSSLURL')->first()->value;

			if (empty($result)) {
				$result = \WHMCS\Database\Capsule::table('tblconfiguration')->where('setting', 'SystemURL')->first()->value;

				if (empty($result)) {
					throw new \Exception('无法从数据库中获取 WHMCS 的地址');
				}
			}
		}
		else if (!empty($GLOBALS['CONFIG']['SystemSSLURL'])) {
			$result = $GLOBALS['CONFIG']['SystemSSLURL'] . '/';
		}
		else if (!!$result = $GLOBALS['CONFIG']['SystemURL']) {
			$result = $GLOBALS['CONFIG']['SystemURL'] . '/';
		}
		else {
			throw new \Exception('无法从全局变量中获取 WHMCS 地址');
		}

		return (string) $result;
	}

	/**
     * 使用 Smarty 模板引擎
     * @param array $template 传入信息包括 tabOverviewReplacementTemplate / templateVariables / directory / cache
     * @return string
     * @throws Exception
     */
	static public function getSmarty(array $template = array())
	{
		$templates_c = (empty($GLOBALS['templates_compiledir']) ? ROOTDIR . '/templates_c' : $GLOBALS['templates_compiledir']);
		if (!is_readable($templates_c) || !is_writeable($templates_c)) {
			throw new \Exception('模板缓存目录 [ ' . $templates_c . ' ] 无法读取或写入，请检查目录权限');
		}

		$templateDirectory = (empty($template['directory']) ? ROOTDIR . '/modules/servers/XingWai/library/theme/' : ROOTDIR . '/' . $template['directory'] . '/');

		if (!file_exists($templateDirectory)) {
			throw new \Exception('模板目录 [ ' . $templateDirectory . ' ] 不存在');
		}

		$smarty = new \Smarty();
		$smarty->compile_dir = $templates_c;

		if ($template['cache']) {
			$smarty->caching = true;
		}
		else {
			$smarty->caching = false;
		}

		$smarty->assign(array('modulelink' => 'addonmodules.php?module=' . $template['module'], 'systemurl' => self::getSystemURL()));

		if (!empty($template['templateVariables'])) {
			if (!is_array($template['templateVariables'])) {
				throw new \Exception('传入信息并非数组');
			}

			$smarty->assign($template['templateVariables']);
		}

		return (string) $smarty->fetch($templateDirectory . $template['tabOverviewReplacementTemplate'] . '.tpl');
	}

	static public function getSmartyTips(array $setting = array())
	{
		$result = self::getSmarty(array(
	'tabOverviewReplacementTemplate' => 'tips',
	'templateVariables'              => array('type' => empty($setting['type']) ? 'success' : $setting['type'], 'title' => $setting['title'], 'message' => $setting['message'], 'noMargin' => $setting['noMargin'], 'canClose' => $setting['canClose'])
	));
		return (string) $result;
	}

	static public function getAdminUser()
	{
		$getInfo = \WHMCS\Database\Capsule::table('tbladmins')->select('username')->first();
		$adminUser = (string) $getInfo->username;

		if (empty($adminUser)) {
			throw new \Exception('无法获取 WHMCS 管理员名称信息');
		}

		return (string) $adminUser;
	}

	static public function getPassword($password = '', $decrypt = true)
	{
		if (empty($password)) {
			throw new \Exception('未定义需要处理的密码内容');
		}

		$values['password2'] = $password;
		$decrypt ? $action = 'decryptpassword' : $action = 'encryptpassword';
		$result = localAPI($action, $values, (string) self::getAdminUser());

		if (!(string) $result['password']) {
			throw new \Exception('密码' . $decrypt ? '解码' : '编码' . '失败');
		}

		return (string) $result['password'];
	}

	static public function getService($serviceid = '')
	{
		return \WHMCS\Database\Capsule::table('tblhosting')->where('id', $serviceid)->first();
	}

	static public function getProduct($packageid = '')
	{
		return \WHMCS\Database\Capsule::table('tblproducts')->where('id', $packageid)->first();
	}

	static public function verifyLicense()
	{

	
	}

	static public function getRate($serverHash = '')
	{
		if (empty($serverHash)) {
			return array();
		}

		$result = explode(',', $serverHash);

		if (count($result) != 8) {
			throw new \Exception('费率格式设置不正确，以逗号分隔、至少 8 位');
		}

		return $result;
	}

	static public function getPeriod(array $vars = array())
	{
		if ($vars['configoption2'] == 'default') {
			$billingCycle = self::getService($vars['serviceid'])->billingcycle;

			if ($vars['configoption4'] == 'host') {
				if (!in_array($billingCycle, array('Annually', 'Biennially', 'Triennially'))) {
					throw new \Exception('虚拟主机仅支持年付');
				}
			}

			$rate = self::getRate($vars['serveraccesshash']);

			switch ($billingCycle) {
			case 'Monthly':
				if (!empty($rate[0])) {
					$result = $rate[0];
				}
				else {
					$result = '0.1';
				}

				break;

			case 'Quarterly':
				if (!empty($rate[1])) {
					$result = $rate[1];
				}
				else {
					$result = '0.3';
				}

				break;

			case 'Semi-Annually':
				if (!empty($rate[2])) {
					$result = $rate[2];
				}
				else {
					$result = '0.6';
				}

				break;

			case 'Annually':
				if (!empty($rate[3])) {
					$result = $rate[3];
				}
				else {
					$result = '1';
				}

				break;

			case 'Biennially':
				if (!empty($rate[4])) {
					$result = $rate[4];
				}
				else {
					$result = '2';
				}

				break;

			case 'Triennially':
				if (!empty($rate[5])) {
					$result = $rate[5];
				}
				else {
					$result = '3';
				}

				break;

			default:
				throw new \Exception('无法获取当前服务的使用周期');
			}

			return $result;
		}

		switch ($vars['configoption2']) {
		case 999:
			$result = 999;
			break;

		case 4:
			if (!empty($rate[6])) {
				$result = $rate[6];
			}
			else {
				$result = 4;
			}

			break;

		case 5:
			if (!empty($rate[7])) {
				$result = $rate[7];
			}
			else {
				$result = 5;
			}

			break;

		default:
			throw new \Exception('无法识别当前所设置的使用周期');
		}

		return $result;
	}

	static public function getResult($string = '')
	{
		$string = explode('&', $string);
		$result = array();

		foreach ($string as $value) {
			$value = explode('=', $value);
			$result[$value[0]] = $value[1];
		}

		if (empty($result)) {
			throw new \Exception('返回的结果为空或格式不正确');
		}

		return $result;
	}
}

require_once dirname(__DIR__) . '/init.php';

?>
